describe("Table", function() {
    describe("openDatabase", function() {
        it("データベースオブジェクトを取得できるべき", function() {
            var db = Table.openDatabase();
            expect(db).toBeDefined();
        });
    });

    // テスト用テーブルの名前を生成
    var tableName = (function() {
        var suffix = String(Math.random() + 10).replace(".", "");
        return "tbl" + suffix;
    })();

    // テスト用のモデルを作成
    var User = Backbone.Model.extend({
        table: new Table(tableName, {
            name: "TEXT",
            age: "INTEGER"
        })
    });

    describe("createTable", function() {
        it("テーブルを作成できるべき", function() {
            var success = false;
            User.prototype.table.createTable({
                success: function(result) {
                    success = true;
                }
            });
            waitsFor(function() {
                return success;
            }, "timeout", 1000);
            runs(function() {
                expect(success).toEqual(true);
            });
        });
    });

    describe("dropTable", function() {
        beforeEach(function() {
            var success = false;
            User.prototype.table.createTable({
                success: function(result) {
                    success = true;
                }
            });
            waitsFor(function() {
                return success;
            }, "timeout", 1000);
        });

        it("テーブルを削除できるべき", function() {
            var success = false;
            User.prototype.table.dropTable({
                success: function(result) {
                    success = true;
                }
            });
            waitsFor(function() {
                return success;
            }, "timeout", 1000);
            runs(function() {
                expect(success).toEqual(true);
            });
        });
    });

    describe("save", function() {
        describe("モデルが存在しないとき", function() {
            beforeEach(function() {
                var success = false;
                User.prototype.table.createTable({
                    success: function(result) {
                        success = true;
                    }
                });
                waitsFor(function() {
                    return success;
                }, "timeout", 1000);
            });

            afterEach(function() {
                var success = false;
                User.prototype.table.dropTable({
                    success: function(result) {
                        success = true;
                    }
                });
                waitsFor(function() {
                    return success;
                }, "timeout", 1000);
            });

            it("モデルを作成できるべき", function() {
                var success = false;
                var response = null;

                var user = new User({ name: "foo", age: 20 });
                user.save(null, {
                    success: function(resp) {
                        success = true;
                        response = resp;
                    }
                });

                waitsFor(function() {
                    return success;
                }, "timeout", 1000);
                
                runs(function() {
                    expect(success).toEqual(true);
                    expect(response.get("name")).toEqual("foo");
                    expect(response.get("age")).toEqual(20);
                });
            });
        });

        describe("モデルが存在するとき", function() {
            beforeEach(function() {
                var success = false;
                User.prototype.table.createTable({
                    success: function(result) {
                        success = true;
                    }
                });
                waitsFor(function() {
                    return success;
                }, "timeout", 1000);

                var created = false;
                var user = new User({ name: "foo", age: 20 });
                user.save(null, {
                    success: function(resp) {
                        created = true;
                    }
                });
                waitsFor(function() {
                    return created;
                }, "timeout", 1000);
            });

            afterEach(function() {
                var success = false;
                User.prototype.table.dropTable({
                    success: function(result) {
                        success = true;
                    }
                });
                waitsFor(function() {
                    return success;
                }, "timeout", 1000);
            });

            it("モデルを更新できるべき", function() {
                var success = false;
                var response = null;

                var user = new User({ name: "bar", age: 30 });
                user.id = 1;
                user.save(null, {
                    success: function(model) {
                        success = true;
                        response = model;
                    }
                });

                waitsFor(function() {
                    return success;
                }, "timeout", 1000);
                
                runs(function() {
                    expect(success).toEqual(true);
                    expect(response.id).toEqual(1);
                    expect(response.get("name")).toEqual("bar");
                    expect(response.get("age")).toEqual(30);
                });
            });
        });
    });

    describe("destroy", function() {
        describe("モデルが存在するとき", function() {
            beforeEach(function() {
                var success = false;
                User.prototype.table.createTable({
                    success: function(result) {
                        success = true;
                    }
                });
                waitsFor(function() {
                    return success;
                }, "timeout", 1000);

                var created = false;
                var user = new User({ name: "foo", age: 20 });
                user.save(null, {
                    success: function(resp) {
                        created = true;
                    }
                });
                waitsFor(function() {
                    return created;
                }, "timeout", 1000);
            });

            afterEach(function() {
                var success = false;
                User.prototype.table.dropTable({
                    success: function(result) {
                        success = true;
                    }
                });
                waitsFor(function() {
                    return success;
                }, "timeout", 1000);
            });

            it("モデルを削除できるべき", function() {
                var success = false;
                var response = null;

                var user = new User();
                user.id = 1;
                user.destroy({
                    success: function(model) {
                        success = true;
                        response = model;
                    }
                });
                
                waitsFor(function() {
                    return success;
                }, "timeout", 1000);

                runs(function() {
                    expect(success).toEqual(true);
                    expect(response.id).toEqual(1);
                });
            });
        });
    });

    describe("fetch", function() {
        describe("モデルが存在するとき", function() {
            beforeEach(function() {
                var success = false;
                User.prototype.table.createTable({
                    success: function(result) {
                        success = true;
                    }
                });
                waitsFor(function() {
                    return success;
                }, "timeout", 1000);

                var created = false;
                var user = new User({ name: "foo", age: 20 });
                user.save(null, {
                    success: function(resp) {
                        created = true;
                    }
                });
                waitsFor(function() {
                    return created;
                }, "timeout", 1000);
            });

            afterEach(function() {
                var success = false;
                User.prototype.table.dropTable({
                    success: function(result) {
                        success = true;
                    }
                });
                waitsFor(function() {
                    return success;
                }, "timeout", 1000);
            });

            it("モデルを取得できるべき", function() {
                var success = false;
                var response = null;

                var user = new User();
                user.id = 1;
                user.fetch({
                    success: function(model) {
                        success = true;
                        response = model;
                    }
                });

                waitsFor(function() {
                    return success;
                }, "timeout", 1000);

                runs(function() {
                    expect(success).toEqual(true);
                    expect(response.id).toEqual(1);
                    expect(response.get("name")).toEqual("foo");
                    expect(response.get("age")).toEqual(20);
                });
            });
        });
    });
    
    describe("fetchAll", function() {
        var UserList = Backbone.Collection.extend({
            model: User
        });

        describe("モデルが存在するとき", function() {
            beforeEach(function() {
                var success = false;
                User.prototype.table.createTable({
                    success: function(result) {
                        success = true;
                    }
                });
                waitsFor(function() {
                    return success;
                }, "timeout", 1000);

                var created1 = false;
                var user1 = new User({ name: "foo", age: 20 });
                user1.save(null, {
                    success: function(resp) {
                        created1 = true;
                    }
                });
                waitsFor(function() {
                    return created1;
                }, "timeout", 1000);

                var created2 = false;
                var user2 = new User({ name: "bar", age: 30 });
                user2.save(null, {
                    success: function(resp) {
                        created2 = true;
                    }
                });
                waitsFor(function() {
                    return created2;
                }, "timeout", 1000);
            });

            afterEach(function() {
                var success = false;
                User.prototype.table.dropTable({
                    success: function(result) {
                        success = true;
                    }
                });
                waitsFor(function() {
                    return success;
                }, "timeout", 1000);
            });

            it("モデルを取得できるべき", function() {
                var success = false;
                var response = null;

                var users = new UserList();
                users.fetch({
                    success: function(models) {
                        success = true;
                        response = models;
                    }
                });

                waitsFor(function() {
                    return success;
                }, "timeout", 1000);
                
                runs(function() {
                    expect(success).toEqual(true);
                    expect(response.length).toEqual(2);
                    expect(users.length).toEqual(2);
                });
            });

            it("フィルタで絞り込んでモデルを取得できるべき", function() {
                var success = false;
                var response = null;

                var users = new UserList();
                users.filterString = "age = ?";
                users.filterParams = [20];
                users.fetch({
                    success: function(models) {
                        success = true;
                        response = models;
                    }
                });

                waitsFor(function() {
                    return success;
                }, "timeout", 1000);
                
                runs(function() {
                    expect(success).toEqual(true);
                    expect(response.length).toEqual(1);
                    expect(users.length).toEqual(1);
                });
            });

            it("指定した並び順で取得できるべき", function() {
                var success = false;
                var response = null;

                var users = new UserList();
                users.orderString = "age DESC";
                users.fetch({
                    success: function(models) {
                        success = true;
                        response = models;
                    }
                });

                waitsFor(function() {
                    return success;
                }, "timeout", 1000);
                
                runs(function() {
                    expect(success).toEqual(true);
                    expect(response.length).toEqual(2);
                    expect(users.length).toEqual(2);
                    expect(users.at(0).get("age")).toEqual(30);
                    expect(users.at(1).get("age")).toEqual(20);
                });
            });

            it("フィルタに該当するモデルがないときは空になるべき", function() {
                var success = false;
                var response = null;

                var users = new UserList();
                users.filterString = "age = 9999";
                users.fetch({
                    success: function(models) {
                        success = true;
                        response = models;
                    }
                });

                waitsFor(function() {
                    return success;
                }, "timeout", 1000);
                
                runs(function() {
                    expect(success).toEqual(true);
                    expect(response.length).toEqual(0);
                    expect(users.length).toEqual(0);
                });
            });
        });
    });
});

